/*
 * q_tbf.c - Tocken Bucket Filer qdisc
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 EPFL-ICA, Network Robots
 */


#include <stddef.h>

#include "error.h"
#include "param.h"
#include "tree.h"
#include "tc.h"
#include "device.h" /* for guess_mtu */
#include "qdisc_common.h"
#include "qdisc.h"


/* ------------------------------- Checking -------------------------------- */


static void tbf_check(QDISC *qdisc)
{
    int guessed_mtu;

    param_get(qdisc->params,qdisc->location);
    guessed_mtu = guess_mtu(qdisc->parent.device);
    if (guessed_mtu > prm_mtu.v)
	warnf("real MTU (guessed %d) may be larger than %d",guessed_mtu,
	  (int) prm_mtu.v);
    if (prm_burst.v < prm_mtu.v)
	lerrorf(qdisc->location,"burst (%d) must be larger than MTU (%d)",
	  (int) prm_burst.v,(int) prm_mtu.v);
/* @@@ check that burst and limit >= rate/HZ */
    if (prm_mpu.present && prm_mpu.v < prm_mtu.v)
	lerrorf(qdisc->location,"MPU (%d) smaller than MTU (%d)",
	  (int) prm_mpu.v,(int) prm_mtu.v);
    if (!prm_rate.v) lerrorf(qdisc->location,"rate must be non-zero");
    if (prm_peakrate.present && prm_rate.v > prm_peakrate.v)
	lerrorf(qdisc->location,"peakrate (%d) must be greater than rate (%d)",
	  (int) prm_peakrate.v,(int) prm_rate.v);
}


/* -------------------------------- Dump tc -------------------------------- */


static void tbf_dump_tc(QDISC *qdisc)
{
    tc_pragma(qdisc->params);
    param_get(qdisc->params,qdisc->location);
    tc_qdisc_add(qdisc);
    tc_add_size("burst",prm_burst.v);
    tc_add_size("limit",prm_limit.v);
    tc_add_size("mtu",prm_mtu.v);
    tc_add_rate("rate",prm_rate.v);
    if (prm_mpu.present) tc_add_size("mpu",prm_mpu.v);
    if (prm_peakrate.present) tc_add_rate("peakrate",prm_peakrate.v);
    tc_nl();
}


/* ------------------------------ Descriptors ------------------------------ */


static const PARAM_DSC *tbf_qdisc_req[] = {
    &prm_rate,		/* rate */
    &prm_burst,		/* size */
    &prm_limit,		/* size; users can calculate latency in tcng */
    &prm_mtu,		/* size (tc sometimes defaults to 2047) */
    NULL
};

static const PARAM_DSC *tbf_qdisc_opt[] = {
    &prm_mpu,		/* size */
    &prm_peakrate,	/* rate */
    &prm_pragma,	/* list */
    NULL
};

static PARAM_DEF tbf_qdisc = {
    .required = tbf_qdisc_req,
    .optional = tbf_qdisc_opt,
};

QDISC_DSC tbf_dsc = {
    .name = "tbf",
    .flags = 0,
    .qdisc_param = &tbf_qdisc,
    .check = &tbf_check,
    .dump_tc = &tbf_dump_tc,
    .dump_ext = &generic_dump_ext,
};
